@@ -19,7 +19,6 @@
 
 #include <gtk/gtk.h>
 #include "gailcontainer.h"
-#include "gailcontainerfactory.h"
 
 static void         gail_container_class_init          (GailContainerClass *klass);
 static void         gail_container_object_init         (GailContainer      *container);
@@ -32,8 +31,8 @@ static gint         gail_container_add_gtk             (GtkContainer       *cont
 static gint         gail_container_remove_gtk          (GtkContainer       *container,
                                                         GtkWidget          *widget);
 
-static void          gail_container_real_init          (GailWidget         *widget,
-                                                        GtkWidget          *gtk_widget);
+static void          gail_container_real_initialize    (AtkObject          *obj,
+                                                        gpointer           data);
 
 static void          gail_container_finalize           (GObject            *object);
 
@@ -72,18 +71,14 @@ gail_container_class_init (GailContainerClass *klass)
 {
   GObjectClass *gobject_class = G_OBJECT_CLASS (klass);
   AtkObjectClass *class = ATK_OBJECT_CLASS (klass);
-  GailWidgetClass *widget_class;
 
-  widget_class = (GailWidgetClass*)klass;
-
-  parent_class = g_type_class_ref (GAIL_TYPE_WIDGET);
+  parent_class = g_type_class_peek_parent (klass);
 
   gobject_class->finalize = gail_container_finalize;
 
   class->get_n_children = gail_container_get_n_children;
   class->ref_child = gail_container_ref_child;
-
-  widget_class->init = gail_container_real_init;
+  class->initialize = gail_container_real_initialize;
 }
 
 static void
@@ -102,9 +97,9 @@ gail_container_new (GtkWidget *widget)
 
   object = g_object_new (GAIL_TYPE_CONTAINER, NULL);
 
-  gail_widget_init (GAIL_WIDGET (object), widget);
-
   accessible = ATK_OBJECT (object);
+  atk_object_initialize (accessible, widget);
+
   if (GTK_IS_TOOLBAR (widget))
     accessible->role = ATK_ROLE_TOOL_BAR;
   else if (GTK_IS_VIEWPORT (widget))
@@ -150,6 +145,11 @@ gail_container_ref_child (AtkObject *obj,
 
   children = gtk_container_children (GTK_CONTAINER(widget));
   tmp_list = g_list_nth (children, i);
+  if (!tmp_list)
+    {
+      g_list_free (children);
+      return NULL;
+    }  
   accessible = gtk_widget_get_accessible (GTK_WIDGET (tmp_list->data));
 
   g_list_free (children);
@@ -204,28 +204,32 @@ gail_container_remove_gtk (GtkContainer       *container,
 }
 
 static void
-gail_container_real_init (GailWidget         *widget,
-                          GtkWidget          *gtk_widget)
+gail_container_real_initialize (AtkObject *obj,
+                                gpointer  data)
 {
-  GailContainer *container = GAIL_CONTAINER (widget);
-  parent_class->init (widget, gtk_widget);
+  GailContainer *container = GAIL_CONTAINER (obj);
+  guint handler_id;
 
-  container->children = gtk_container_children (GTK_CONTAINER (gtk_widget));
+  ATK_OBJECT_CLASS (parent_class)->initialize (obj, data);
+
+  container->children = gtk_container_children (GTK_CONTAINER (data));
 
   /*
    * We store the handler ids for these signals as some objects, e.g.
    * GailButton need to remove these handlers.
    */
-  container->add_handler =
-      g_signal_connect (gtk_widget,
-                        "add",
-                        G_CALLBACK (gail_container_add_gtk),
-                        NULL);
-  container->remove_handler =
-      g_signal_connect (gtk_widget,
-                        "remove",
-                        G_CALLBACK (gail_container_remove_gtk),
-                        NULL);
+  handler_id = g_signal_connect (data,
+                                 "add",
+                                 G_CALLBACK (gail_container_add_gtk),
+                                 NULL);
+  g_object_set_data (G_OBJECT (obj), "gail-add-handler-id", 
+                     GUINT_TO_POINTER (handler_id));
+  handler_id = g_signal_connect (data,
+                                 "remove",
+                                 G_CALLBACK (gail_container_remove_gtk),
+                                 NULL);
+  g_object_set_data (G_OBJECT (obj), "gail-remove-handler-id", 
+                     GUINT_TO_POINTER (handler_id));
 }
 
 static void
