@@ -20,7 +20,6 @@
 #include <string.h>
 #include <gtk/gtk.h>
 #include "gaillabel.h"
-#include "gaillabelfactory.h"
 #include <util/gailmisc.h>
 
 static void       gail_label_class_init            (GailLabelClass    *klass);
@@ -37,7 +36,6 @@ static void       atk_text_interface_init          (AtkTextIface      *iface);
 static G_CONST_RETURN gchar* gail_label_get_name         (AtkObject         *accessible);
 static AtkStateSet*          gail_label_ref_state_set	 (AtkObject	    *accessible);
 static AtkRelationSet*       gail_label_ref_relation_set (AtkObject         *accessible);
-;
 
 /* atktext.h */
 
