@@ -219,6 +219,8 @@ atk_key_event_from_gdk_event_key (GdkEventKey *key)
 	    event->type = ATK_KEY_EVENT_RELEASE;
 	    break;
     default:
+	    g_assert_not_reached ();
+	    return NULL;
     }
   event->state = key->state;
   event->keyval = key->keyval;
